/*
Author: Hans Summers, 2015
Website: http://www.hanssummers.com

A very very simple Si5351a demonstration
using the Si5351a module kit http://www.hanssummers.com/synth
Please also refer to SiLabs AN619 which describes all the registers to use
*/

#include <inttypes.h>
#include "Main.h"
#include "i2c.h"
#include "si5351a.h"

/*
    PLL  mult, num  denom
mult   15..90
num   0..1,048,575 (0xFFFFF)
denom   0..1,048,575 (0xFFFFF)
*/
void setupPLL(uint8_t pll, uint8_t mult, uint32_t num, uint32_t denom)
{
	uint32_t P1;					// PLL   P1
	uint32_t P2;					// PLL   P2
	uint32_t P3;					// PLL   P3

	P1 = (uint32_t)(128 * ((float)num / (float)denom));
	P1 = (uint32_t)(128 * (uint32_t)(mult) + P1 - 512);
	P2 = (uint32_t)(128 * ((float)num / (float)denom));
	P2 = (uint32_t)(128 * num - denom * P2);
	P3 = denom;

	i2cSendRegister(pll + 0, (P3 & 0x0000FF00) >> 8);
	i2cSendRegister(pll + 1, (P3 & 0x000000FF));
	i2cSendRegister(pll + 2, (P1 & 0x00030000) >> 16);
	i2cSendRegister(pll + 3, (P1 & 0x0000FF00) >> 8);
	i2cSendRegister(pll + 4, (P1 & 0x000000FF));
	i2cSendRegister(pll + 5, ((P3 & 0x000F0000) >> 12) | ((P2 & 0x000F0000) >> 16));
	i2cSendRegister(pll + 6, (P2 & 0x0000FF00) >> 8);
	i2cSendRegister(pll + 7, (P2 & 0x000000FF));
}

/*
 MultiSynth     R- 
R- -   ,     , . #define  si5351a.h
*/
void setupMultisynth(uint8_t synth, uint32_t divider, uint8_t rDiv)
{
	uint32_t P1;
	uint32_t P2;
	uint32_t P3;

	P1 = 128 * divider - 512;
	P2 = 0;							// P2 = 0, P3 = 1     
	P3 = 1;

	i2cSendRegister(synth + 0,   (P3 & 0x0000FF00) >> 8);
	i2cSendRegister(synth + 1,   (P3 & 0x000000FF));
	i2cSendRegister(synth + 2,   ((P1 & 0x00030000) >> 16) | rDiv);
	i2cSendRegister(synth + 3,   (P1 & 0x0000FF00) >> 8);
	i2cSendRegister(synth + 4,   (P1 & 0x000000FF));
	i2cSendRegister(synth + 5,   ((P3 & 0x000F0000) >> 12) | ((P2 & 0x000F0000) >> 16));
	i2cSendRegister(synth + 6,   (P2 & 0x0000FF00) >> 8);
	i2cSendRegister(synth + 7,   (P2 & 0x000000FF));
}

/*
 AN619    - 0x80   
*/
void si5351aOutputOff(uint8_t clk)
{
	i2cSendRegister(clk, 0x80);
}

/*
Set CLK0 output ON and to the specified frequency
Frequency is in the range 1MHz to 150MHz
Example: si5351aSetFrequency(10000000);
will set output CLK0 to 10MHz

This example sets up PLL A
and MultiSynth 0
and produces the output on CLK0
*/
void si5351aSetFrequency(uint32_t frequency)
{
	uint32_t pllFreq;
	uint32_t xtalFreq = XTAL_FREQ;
	uint32_t l;
	float f;
	uint8_t mult;
	uint32_t num;
	uint32_t denom;
	uint32_t divider;

	/*  . 900,000,000 Hz -    PLLA*/
	divider = FREQ_PLLA / frequency;
	
	/* */
	if (divider % 2) divider--;

	/*  pllFreq:  *   */
	pllFreq = divider * frequency;

	/*     pllFreq
	   :
	1: mult -  ,      15..90
	2: num  denom -  ,   
					    20 bits ( 0..1048575)
	3:   mult + num / denom
	          1048575*/
	mult = pllFreq / xtalFreq;
	l = pllFreq % xtalFreq;
	f = l;
	f *= 1048575;
	f /= xtalFreq;
	num = f;
	denom = 1048575;

	/* PLLA    */
	setupPLL(SI_SYNTH_PLL_A, mult, num, denom);
	
	/*  MultiSynth 0   
	   R     2,   1..128.
	  SI_R_DIV1....SI_R_DIV128 (.   si5351a.h)
	      1 ,    SI_R_DIV128*/
	setupMultisynth(SI_SYNTH_MS_0, divider, SI_R_DIV_1);
	
	/* PLL     ,    
	,    PLL     */
	//i2cSendRegister(SI_PLL_RESET, 0xA0);	
	
	/*  CLK0 (0x4F)
	     MultiSynth0  PLLA*/
	i2cSendRegister(SI_CLK0_CONTROL, 0x4F | SI_CLK_SRC_PLL_A);
}